// 
// Engine.cs
//  
// Author:
//       Michael Hutchinson <mhutchinson@novell.com>
// 
// Copyright (c) 2009 Novell, Inc. (http://www.novell.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using System.CodeDom;
using System.CodeDom.Compiler;
using Mono.TextTemplating;

namespace Microsoft.VisualStudio.TextTemplating
{
	
	
	public class Engine : ITextTemplatingEngine
	{
		
		public Engine ()
		{
		}
		
		public string ProcessTemplate (string content, ITextTemplatingEngineHost host)
		{
			AppDomain appdomain = host.ProvideTemplatingAppDomain (content);
			ITextTemplatingEngine engine;
			if (appdomain != null) {
				engine = (ITextTemplatingEngine)
					appdomain.CreateInstanceAndUnwrap (typeof (TemplatingEngine).Assembly.FullName,
					                                   typeof (TemplatingEngine).FullName);
			} else {
				engine = new TemplatingEngine ();
			}
			
			return engine.ProcessTemplate (content, host);
		}
	}
	
	public interface ITextTemplatingEngine
	{
		string ProcessTemplate (string content, ITextTemplatingEngineHost host);
	}
}
